@@ -31,13 +31,13 @@ const char *ascii_logo =
 "                _._                                                  \n"
 "           _.-``__ ''-._                                             \n"
 "      _.-``    `.  `_.  ''-._           Redis %s (%s/%d) %s bit\n"
-"  .-`` .-```.  ```\\/    _.,_ ''-._                                   \n"
+"  .-`` .-```.  ```\\/    _.,_ ''-._                                  \n"
 " (    '      ,       .-`  | `,    )     Running in %s mode\n"
 " |`-._`-...-` __...-.``-._|'` _.-'|     Port: %d\n"
 " |    `-._   `._    /     _.-'    |     PID: %ld\n"
 "  `-._    `-._  `-./  _.-'    _.-'                                   \n"
 " |`-._`-._    `-.__.-'    _.-'_.-'|                                  \n"
-" |    `-._`-._        _.-'_.-'    |           http://redis.io        \n"
+" |    `-._`-._        _.-'_.-'    |           https://redis.io       \n"
 "  `-._    `-._`-.__.-'_.-'    _.-'                                   \n"
 " |`-._`-._    `-.__.-'    _.-'_.-'|                                  \n"
 " |    `-._`-._        _.-'_.-'    |                                  \n"
