@@ -10,7 +10,7 @@ source "../../../tests/support/cli.tcl"
 # Create a cluster with 5 master and 15 slaves, to make sure there are no
 # empty masters and make rebalancing simpler to handle during the test.
 test "Create a 5 nodes cluster" {
-    create_cluster 5 15
+    cluster_create_with_continuous_slots 5 15
 }
 
 test "Cluster is up" {
@@ -21,7 +21,7 @@ test "Each master should have at least two replicas attached" {
     foreach_redis_id id {
         if {$id < 5} {
             wait_for_condition 1000 50 {
-                [llength [lindex [R 0 role] 2]] >= 2
+                [llength [lindex [R $id role] 2]] >= 2
             } else {
                 fail "Master #$id does not have 2 slaves as expected"
             }
@@ -29,6 +29,12 @@ test "Each master should have at least two replicas attached" {
     }
 }
 
+test "Set allow-replica-migration yes" {
+    foreach_redis_id id {
+        R $id CONFIG SET cluster-allow-replica-migration yes
+    }
+}
+
 set master0_id [dict get [get_myself 0] id]
 test "Resharding all the master #0 slots away from it" {
     set output [exec \
@@ -39,11 +45,12 @@ test "Resharding all the master #0 slots away from it" {
 
 }
 
-test "Master #0 should lose its replicas" {
+test "Master #0 who lost all slots should turn into a replica without replicas" {
     wait_for_condition 1000 50 {
-        [llength [lindex [R 0 role] 2]] == 0
+        [RI 0 role] == "slave" && [RI 0 connected_slaves] == 0
     } else {
-        fail "Master #0 still has replicas"
+        puts [R 0 info replication]
+        fail "Master #0 didn't turn itself into a replica"
     }
 }
 
