@@ -1,9 +1,18 @@
-tags {"rdb"} {
+tags {"rdb external:skip"} {
 
 set server_path [tmpdir "server.rdb-encoding-test"]
 
 # Copy RDB with different encodings in server path
 exec cp tests/assets/encodings.rdb $server_path
+exec cp tests/assets/list-quicklist.rdb $server_path
+
+start_server [list overrides [list "dir" $server_path "dbfilename" "list-quicklist.rdb"]] {
+    test "test old version rdb file" {
+        r select 0
+        assert_equal [r get x] 7
+        r lpop list
+    } {7}
+}
 
 start_server [list overrides [list "dir" $server_path "dbfilename" "encodings.rdb"]] {
   test "RDB encoding loading test" {
@@ -29,7 +38,7 @@ set server_path [tmpdir "server.rdb-startup-test"]
 
 start_server [list overrides [list "dir" $server_path] keep_persistence true] {
     test {Server started empty with non-existing RDB file} {
-        r debug digest
+        debug_digest
     } {0000000000000000000000000000000000000000}
     # Save an RDB file, needed for the next test.
     r save
@@ -37,7 +46,7 @@ start_server [list overrides [list "dir" $server_path] keep_persistence true] {
 
 start_server [list overrides [list "dir" $server_path] keep_persistence true] {
     test {Server started empty with empty RDB file} {
-        r debug digest
+        debug_digest
     } {0000000000000000000000000000000000000000}
 }
 
@@ -54,16 +63,16 @@ start_server [list overrides [list "dir" $server_path] keep_persistence true] {
         set records [r xreadgroup GROUP mygroup Alice COUNT 2 STREAMS stream >]
         r xdel stream [lindex [lindex [lindex [lindex $records 0] 1] 1] 0]
         r xack stream mygroup [lindex [lindex [lindex [lindex $records 0] 1] 0] 0]
-        set digest [r debug digest]
+        set digest [debug_digest]
         r config set sanitize-dump-payload no
         r debug reload
-        set newdigest [r debug digest]
+        set newdigest [debug_digest]
         assert {$digest eq $newdigest}
     }
     test {Test RDB stream encoding - sanitize dump} {
         r config set sanitize-dump-payload yes
         r debug reload
-        set newdigest [r debug digest]
+        set newdigest [debug_digest]
         assert {$digest eq $newdigest}
     }
     # delete the stream, maybe valgrind will find something
@@ -130,19 +139,21 @@ start_server_and_kill_it [list "dir" $server_path] {
 
 start_server {} {
     test {Test FLUSHALL aborts bgsave} {
-        # 1000 keys with 1ms sleep per key shuld take 1 second
+        r config set save ""
+        # 5000 keys with 1ms sleep per key should take 5 second
         r config set rdb-key-save-delay 1000
-        r debug populate 1000
+        populate 5000
+        assert_lessthan 999 [s rdb_changes_since_last_save]
         r bgsave
         assert_equal [s rdb_bgsave_in_progress] 1
         r flushall
-        # wait half a second max
-        wait_for_condition 5 100 {
+        # wait a second max (bgsave should take 5)
+        wait_for_condition 10 100 {
             [s rdb_bgsave_in_progress] == 0
         } else {
             fail "bgsave not aborted"
         }
-        # veirfy that bgsave failed, by checking that the change counter is still high
+        # verify that bgsave failed, by checking that the change counter is still high
         assert_lessthan 999 [s rdb_changes_since_last_save]
         # make sure the server is still writable
         r set x xx
@@ -161,10 +172,10 @@ start_server {} {
 }
 
 test {client freed during loading} {
-    start_server [list overrides [list key-load-delay 10 rdbcompression no]] {
+    start_server [list overrides [list key-load-delay 50 loading-process-events-interval-bytes 1024 rdbcompression no]] {
         # create a big rdb that will take long to load. it is important
         # for keys to be big since the server processes events only once in 2mb.
-        # 100mb of rdb, 100k keys will load in more than 1 second
+        # 100mb of rdb, 100k keys will load in more than 5 seconds
         r debug populate 100000 key 1000
 
         restart_server 0 false false
@@ -172,9 +183,9 @@ test {client freed during loading} {
         # make sure it's still loading
         assert_equal [s loading] 1
 
-        # connect and disconnect 10 clients
+        # connect and disconnect 5 clients
         set clients {}
-        for {set j 0} {$j < 10} {incr j} {
+        for {set j 0} {$j < 5} {incr j} {
             lappend clients [redis_deferring_client]
         }
         foreach rd $clients {
@@ -202,9 +213,34 @@ test {client freed during loading} {
     }
 }
 
+start_server {} {
+    test {Test RDB load info} {
+        r debug populate 1000
+        r save
+        restart_server 0 true false
+        wait_done_loading r
+        assert {[s rdb_last_load_keys_expired] == 0}
+        assert {[s rdb_last_load_keys_loaded] == 1000}
+
+        r debug set-active-expire 0
+        for {set j 0} {$j < 1024} {incr j} {
+            r select [expr $j%16]
+            r set $j somevalue px 10
+        }
+        after 20
+
+        r save
+        restart_server 0 true false
+        wait_done_loading r
+        assert {[s rdb_last_load_keys_expired] == 1024}
+        assert {[s rdb_last_load_keys_loaded] == 1000}
+    }
+}
+
 # Our COW metrics (Private_Dirty) work only on Linux
 set system_name [string tolower [exec uname -s]]
-if {$system_name eq {linux}} {
+set page_size [exec getconf PAGESIZE]
+if {$system_name eq {linux} && $page_size == 4096} {
 
 start_server {overrides {save ""}} {
     test {Test child sending info} {
@@ -221,9 +257,11 @@ start_server {overrides {save ""}} {
         r config set rdb-key-save-delay 200
         r config set loglevel debug
 
-        # populate the db with 10k keys of 4k each
+        # populate the db with 10k keys of 512B each (since we want to measure the COW size by
+        # changing some keys and read the reported COW size, we are using small key size to prevent from
+        # the "dismiss mechanism" free memory and reduce the COW size)
         set rd [redis_deferring_client 0]
-        set size 4096
+        set size 500 ;# aim for the 512 bin (sds overhead)
         set cmd_count 10000
         for {set k 0} {$k < $cmd_count} {incr k} {
             $rd set key$k [string repeat A $size]
@@ -240,12 +278,13 @@ start_server {overrides {save ""}} {
 
         set current_save_keys_total [s current_save_keys_total]
         if {$::verbose} {
-            puts "Keys before bgsave start: current_save_keys_total"
+            puts "Keys before bgsave start: $current_save_keys_total"
         }
 
         # on each iteration, we will write some key to the server to trigger copy-on-write, and
         # wait to see that it reflected in INFO.
         set iteration 1
+        set key_idx 0
         while 1 {
             # take samples before writing new data to the server
             set cow_size [s current_cow_size]
@@ -259,12 +298,19 @@ start_server {overrides {save ""}} {
             }
 
             # trigger copy-on-write
-            r setrange key$iteration 0 [string repeat B $size]
+            set modified_keys 16
+            for {set k 0} {$k < $modified_keys} {incr k} {
+                r setrange key$key_idx 0 [string repeat B $size]
+                incr key_idx 1
+            }
 
+            # changing 16 keys (512B each) will create at least 8192 COW (2 pages), but we don't want the test
+            # to be too strict, so we check for a change of at least 4096 bytes
+            set exp_cow [expr $cow_size + 4096]
             # wait to see that current_cow_size value updated (as long as the child is in progress)
             wait_for_condition 80 100 {
                 [s rdb_bgsave_in_progress] == 0 ||
-                [s current_cow_size] >= $cow_size + $size && 
+                [s current_cow_size] >= $exp_cow &&
                 [s current_save_keys_processed] > $keys_processed &&
                 [s current_fork_perc] > 0
             } else {
@@ -311,4 +357,58 @@ start_server {overrides {save ""}} {
 }
 } ;# system_name
 
+exec cp -f tests/assets/scriptbackup.rdb $server_path
+start_server [list overrides [list "dir" $server_path "dbfilename" "scriptbackup.rdb" "appendonly" "no"]] {
+    # the script is: "return redis.call('set', 'foo', 'bar')""
+    # its sha1   is: a0c38691e9fffe4563723c32ba77a34398e090e6
+    test {script won't load anymore if it's in rdb} {
+        assert_equal [r script exists a0c38691e9fffe4563723c32ba77a34398e090e6] 0
+    }
+}
+
+start_server {} {
+    test "failed bgsave prevents writes" {
+        r config set rdb-key-save-delay 10000000
+        populate 1000
+        r set x x
+        r bgsave
+        set pid1 [get_child_pid 0]
+        catch {exec kill -9 $pid1}
+        waitForBgsave r
+
+        # make sure a read command succeeds
+        assert_equal [r get x] x
+
+        # make sure a write command fails
+        assert_error {MISCONF *} {r set x y}
+
+        # repeate with script
+        assert_error {MISCONF *} {r eval {
+            return redis.call('set','x',1)
+            } 1 x
+        }
+        assert_equal {x} [r eval {
+            return redis.call('get','x')
+            } 1 x
+        ]
+
+        # again with script using shebang
+        assert_error {MISCONF *} {r eval {#!lua
+            return redis.call('set','x',1)
+            } 1 x
+        }
+        assert_equal {x} [r eval {#!lua flags=no-writes
+            return redis.call('get','x')
+            } 1 x
+        ]
+
+        r config set rdb-key-save-delay 0
+        r bgsave
+        waitForBgsave r
+
+        # server is writable again
+        r set x y
+    } {OK}
+}
+
 } ;# tags
