@@ -1,4 +1,4 @@
-start_server {tags {"aofrw"}} {
+start_server {tags {"aofrw external:skip"}} {
     # Enable the AOF
     r config set appendonly yes
     r config set auto-aof-rewrite-percentage 0 ; # Disable auto-rewrite.
@@ -45,11 +45,11 @@ start_server {tags {"aofrw"}} {
             wait_load_handlers_disconnected
 
             # Get the data set digest
-            set d1 [r debug digest]
+            set d1 [debug_digest]
 
             # Load the AOF
             r debug loadaof
-            set d2 [r debug digest]
+            set d2 [debug_digest]
 
             # Make sure they are the same
             assert {$d1 eq $d2}
@@ -57,19 +57,24 @@ start_server {tags {"aofrw"}} {
     }
 }
 
-start_server {tags {"aofrw"} overrides {aof-use-rdb-preamble no}} {
+start_server {tags {"aofrw external:skip"} overrides {aof-use-rdb-preamble no}} {
     test {Turning off AOF kills the background writing child if any} {
         r config set appendonly yes
         waitForBgrewriteaof r
-        r multi
+
+        # start a slow AOFRW
+        r set k v
+        r config set rdb-key-save-delay 10000000
         r bgrewriteaof
+
+        # disable AOF and wait for the child to be killed
         r config set appendonly no
-        r exec
         wait_for_condition 50 100 {
             [string match {*Killing*AOF*child*} [exec tail -5 < [srv 0 stdout]]]
         } else {
             fail "Can't find 'Killing AOF child' into recent logs"
         }
+        r config set rdb-key-save-delay 0
     }
 
     foreach d {string int} {
@@ -86,11 +91,11 @@ start_server {tags {"aofrw"} overrides {aof-use-rdb-preamble no}} {
                     r lpush key $data
                 }
                 assert_equal [r object encoding key] $e
-                set d1 [r debug digest]
+                set d1 [debug_digest]
                 r bgrewriteaof
                 waitForBgrewriteaof r
                 r debug loadaof
-                set d2 [r debug digest]
+                set d2 [debug_digest]
                 if {$d1 ne $d2} {
                     error "assertion:$d1 is not equal to $d2"
                 }
@@ -114,11 +119,11 @@ start_server {tags {"aofrw"} overrides {aof-use-rdb-preamble no}} {
                 if {$d ne {string}} {
                     assert_equal [r object encoding key] $e
                 }
-                set d1 [r debug digest]
+                set d1 [debug_digest]
                 r bgrewriteaof
                 waitForBgrewriteaof r
                 r debug loadaof
-                set d2 [r debug digest]
+                set d2 [debug_digest]
                 if {$d1 ne $d2} {
                     error "assertion:$d1 is not equal to $d2"
                 }
@@ -127,10 +132,10 @@ start_server {tags {"aofrw"} overrides {aof-use-rdb-preamble no}} {
     }
 
     foreach d {string int} {
-        foreach e {ziplist hashtable} {
+        foreach e {listpack hashtable} {
             test "AOF rewrite of hash with $e encoding, $d data" {
                 r flushall
-                if {$e eq {ziplist}} {set len 10} else {set len 1000}
+                if {$e eq {listpack}} {set len 10} else {set len 1000}
                 for {set j 0} {$j < $len} {incr j} {
                     if {$d eq {string}} {
                         set data [randstring 0 16 alpha]
@@ -140,11 +145,11 @@ start_server {tags {"aofrw"} overrides {aof-use-rdb-preamble no}} {
                     r hset key $data $data
                 }
                 assert_equal [r object encoding key] $e
-                set d1 [r debug digest]
+                set d1 [debug_digest]
                 r bgrewriteaof
                 waitForBgrewriteaof r
                 r debug loadaof
-                set d2 [r debug digest]
+                set d2 [debug_digest]
                 if {$d1 ne $d2} {
                     error "assertion:$d1 is not equal to $d2"
                 }
@@ -153,10 +158,10 @@ start_server {tags {"aofrw"} overrides {aof-use-rdb-preamble no}} {
     }
 
     foreach d {string int} {
-        foreach e {ziplist skiplist} {
+        foreach e {listpack skiplist} {
             test "AOF rewrite of zset with $e encoding, $d data" {
                 r flushall
-                if {$e eq {ziplist}} {set len 10} else {set len 1000}
+                if {$e eq {listpack}} {set len 10} else {set len 1000}
                 for {set j 0} {$j < $len} {incr j} {
                     if {$d eq {string}} {
                         set data [randstring 0 16 alpha]
@@ -166,11 +171,11 @@ start_server {tags {"aofrw"} overrides {aof-use-rdb-preamble no}} {
                     r zadd key [expr rand()] $data
                 }
                 assert_equal [r object encoding key] $e
-                set d1 [r debug digest]
+                set d1 [debug_digest]
                 r bgrewriteaof
                 waitForBgrewriteaof r
                 r debug loadaof
-                set d2 [r debug digest]
+                set d2 [debug_digest]
                 if {$d1 ne $d2} {
                     error "assertion:$d1 is not equal to $d2"
                 }
@@ -178,29 +183,42 @@ start_server {tags {"aofrw"} overrides {aof-use-rdb-preamble no}} {
         }
     }
 
-    test {BGREWRITEAOF is delayed if BGSAVE is in progress} {
-        r multi
-        r bgsave
+    test "AOF rewrite functions" {
+        r flushall
+        r FUNCTION LOAD {#!lua name=test
+            redis.register_function('test', function() return 1 end)
+        }
         r bgrewriteaof
-        r info persistence
-        set res [r exec]
-        assert_match {*scheduled*} [lindex $res 1]
-        assert_match {*aof_rewrite_scheduled:1*} [lindex $res 2]
-        while {[string match {*aof_rewrite_scheduled:1*} [r info persistence]]} {
+        waitForBgrewriteaof r
+        r function flush
+        r debug loadaof
+        assert_equal [r fcall test 0] 1
+        r FUNCTION LIST
+    } {{library_name test engine LUA functions {{name test description {} flags {}}}}}
+
+    test {BGREWRITEAOF is delayed if BGSAVE is in progress} {
+        r flushall
+        r set k v
+        r config set rdb-key-save-delay 10000000
+        r bgsave
+        assert_match {*scheduled*} [r bgrewriteaof]
+        assert_equal [s aof_rewrite_scheduled] 1
+        r config set rdb-key-save-delay 0
+        catch {exec kill -9 [get_child_pid 0]}
+        while {[s aof_rewrite_scheduled] eq 1} {
             after 100
         }
     }
 
     test {BGREWRITEAOF is refused if already in progress} {
+        r config set aof-use-rdb-preamble yes
+        r config set rdb-key-save-delay 10000000
         catch {
-            r multi
             r bgrewriteaof
             r bgrewriteaof
-            r exec
         } e
         assert_match {*ERR*already*} $e
-        while {[string match {*aof_rewrite_scheduled:1*} [r info persistence]]} {
-            after 100
-        }
+        r config set rdb-key-save-delay 0
+        catch {exec kill -9 [get_child_pid 0]}
     }
 }
