@@ -12,7 +12,7 @@ start_server {tags {"dump"}} {
         r del foo
         r restore foo 5000 $encoded
         set ttl [r pttl foo]
-        assert {$ttl >= 3000 && $ttl <= 5000}
+        assert_range $ttl 3000 5000
         r get foo
     } {bar}
 
@@ -22,7 +22,7 @@ start_server {tags {"dump"}} {
         r del foo
         r restore foo 2569591501 $encoded
         set ttl [r pttl foo]
-        assert {$ttl >= (2569591501-3000) && $ttl <= 2569591501}
+        assert_range $ttl (2569591501-3000) 2569591501
         r get foo
     } {bar}
     
@@ -33,7 +33,7 @@ start_server {tags {"dump"}} {
         set now [clock milliseconds]
         r restore foo [expr $now+3000] $encoded absttl
         set ttl [r pttl foo]
-        assert {$ttl >= 2900 && $ttl <= 3100}
+        assert_range $ttl 2000 3100
         r get foo
     } {bar}
 
@@ -46,7 +46,7 @@ start_server {tags {"dump"}} {
         catch {r debug object foo} e
         r debug set-active-expire 1
         set e
-    } {ERR no such key}
+    } {ERR no such key} {needs:debug}
 
     test {RESTORE can set LRU} {
         r set foo bar
@@ -56,8 +56,9 @@ start_server {tags {"dump"}} {
         r restore foo 0 $encoded idletime 1000
         set idle [r object idletime foo]
         assert {$idle >= 1000 && $idle <= 1010}
-        r get foo
-    } {bar}
+        assert_equal [r get foo] {bar}
+        r config set maxmemory-policy noeviction
+    } {OK} {needs:config-maxmemory}
     
     test {RESTORE can set LFU} {
         r set foo bar
@@ -68,7 +69,9 @@ start_server {tags {"dump"}} {
         set freq [r object freq foo]
         assert {$freq == 100}
         r get foo
-    } {bar}
+        assert_equal [r get foo] {bar}
+        r config set maxmemory-policy noeviction
+    } {OK} {needs:config-maxmemory}
 
     test {RESTORE returns an error of the key already exists} {
         r set foo bar
@@ -111,7 +114,7 @@ start_server {tags {"dump"}} {
             r -1 migrate $second_host $second_port key 9 1000
             assert_match {*migrate_cached_sockets:1*} [r -1 info]
         }
-    }
+    } {} {external:skip}
 
     test {MIGRATE cached connections are released after some time} {
         after 15000
@@ -135,7 +138,7 @@ start_server {tags {"dump"}} {
             assert {[$second get key] eq {Some Value}}
             assert {[$second ttl key] == -1}
         }
-    }
+    } {} {external:skip}
 
     test {MIGRATE is able to copy a key between two instances} {
         set first [srv 0 client]
@@ -154,7 +157,7 @@ start_server {tags {"dump"}} {
             assert {[$second exists list] == 1}
             assert {[$first lrange list 0 -1] eq [$second lrange list 0 -1]}
         }
-    }
+    } {} {external:skip}
 
     test {MIGRATE will not overwrite existing keys, unless REPLACE is used} {
         set first [srv 0 client]
@@ -170,13 +173,13 @@ start_server {tags {"dump"}} {
             $second set list somevalue
             catch {r -1 migrate $second_host $second_port list 9 5000 copy} e
             assert_match {ERR*} $e
-            set res [r -1 migrate $second_host $second_port list 9 5000 copy replace]
+            set ret [r -1 migrate $second_host $second_port list 9 5000 copy replace]
             assert {$ret eq {OK}}
             assert {[$first exists list] == 1}
             assert {[$second exists list] == 1}
             assert {[$first lrange list 0 -1] eq [$second lrange list 0 -1]}
         }
-    }
+    } {} {external:skip}
 
     test {MIGRATE propagates TTL correctly} {
         set first [srv 0 client]
@@ -196,7 +199,7 @@ start_server {tags {"dump"}} {
             assert {[$second get key] eq {Some Value}}
             assert {[$second ttl key] >= 7 && [$second ttl key] <= 10}
         }
-    }
+    } {} {external:skip}
 
     test {MIGRATE can correctly transfer large values} {
         set first [srv 0 client]
@@ -221,7 +224,7 @@ start_server {tags {"dump"}} {
             assert {[$second ttl key] == -1}
             assert {[$second llen key] == 40000*20}
         }
-    }
+    } {} {external:skip}
 
     test {MIGRATE can correctly transfer hashes} {
         set first [srv 0 client]
@@ -241,7 +244,7 @@ start_server {tags {"dump"}} {
             assert {[$second exists key] == 1}
             assert {[$second ttl key] == -1}
         }
-    }
+    } {} {external:skip}
 
     test {MIGRATE timeout actually works} {
         set first [srv 0 client]
@@ -260,7 +263,7 @@ start_server {tags {"dump"}} {
             catch {r -1 migrate $second_host $second_port key 9 500} e
             assert_match {IOERR*} $e
         }
-    }
+    } {} {external:skip}
 
     test {MIGRATE can migrate multiple keys at once} {
         set first [srv 0 client]
@@ -283,12 +286,12 @@ start_server {tags {"dump"}} {
             assert {[$second get key2] eq {v2}}
             assert {[$second get key3] eq {v3}}
         }
-    }
+    } {} {external:skip}
 
     test {MIGRATE with multiple keys must have empty key arg} {
         catch {r MIGRATE 127.0.0.1 6379 NotEmpty 9 5000 keys a b c} e
         set e
-    } {*empty string*}
+    } {*empty string*} {external:skip}
 
     test {MIGRATE with multiple keys migrate just existing ones} {
         set first [srv 0 client]
@@ -314,7 +317,7 @@ start_server {tags {"dump"}} {
             assert {[$second get key2] eq {v2}}
             assert {[$second get key3] eq {v3}}
         }
-    }
+    } {} {external:skip}
 
     test {MIGRATE with multiple keys: stress command rewriting} {
         set first [srv 0 client]
@@ -330,7 +333,7 @@ start_server {tags {"dump"}} {
             assert {[$first dbsize] == 0}
             assert {[$second dbsize] == 15}
         }
-    }
+    } {} {external:skip}
 
     test {MIGRATE with multiple keys: delete just ack keys} {
         set first [srv 0 client]
@@ -350,7 +353,7 @@ start_server {tags {"dump"}} {
             assert {[$first exists c] == 1}
             assert {[$first exists d] == 1}
         }
-    }
+    } {} {external:skip}
 
     test {MIGRATE AUTH: correct and wrong password cases} {
         set first [srv 0 client]
@@ -375,5 +378,5 @@ start_server {tags {"dump"}} {
             catch {r -1 migrate $second_host $second_port list 9 5000 AUTH foobar} err
             assert_match {*WRONGPASS*} $err
         }
-    }
+    } {} {external:skip}
 }
