@@ -21,7 +21,7 @@ proc test_memory_efficiency {range} {
     return $efficiency
 }
 
-start_server {tags {"memefficiency"}} {
+start_server {tags {"memefficiency external:skip"}} {
     foreach {size_range expected_min_efficiency} {
         32    0.15
         64    0.25
@@ -37,7 +37,7 @@ start_server {tags {"memefficiency"}} {
 }
 
 run_solo {defrag} {
-start_server {tags {"defrag"} overrides {appendonly yes auto-aof-rewrite-percentage 0 save ""}} {
+start_server {tags {"defrag external:skip"} overrides {appendonly yes auto-aof-rewrite-percentage 0 save ""}} {
     if {[string match {*jemalloc*} [s mem_allocator]] && [r debug mallctl arenas.page] <= 8192} {
         test "Active defrag" {
             r config set hz 100
@@ -61,7 +61,7 @@ start_server {tags {"defrag"} overrides {appendonly yes auto-aof-rewrite-percent
             r config set latency-monitor-threshold 5
             r latency reset
             r config set maxmemory 110mb ;# prevent further eviction (not to fail the digest test)
-            set digest [r debug digest]
+            set digest [debug_digest]
             catch {r config set activedefrag yes} e
             if {[r config get activedefrag] eq "activedefrag yes"} {
                 # Wait for the active defrag to start working (decision once a
@@ -73,7 +73,7 @@ start_server {tags {"defrag"} overrides {appendonly yes auto-aof-rewrite-percent
                 }
 
                 # Wait for the active defrag to stop working.
-                wait_for_condition 150 100 {
+                wait_for_condition 2000 100 {
                     [s active_defrag_running] eq 0
                 } else {
                     after 120 ;# serverCron only updates the info once in 100ms
@@ -82,7 +82,7 @@ start_server {tags {"defrag"} overrides {appendonly yes auto-aof-rewrite-percent
                     fail "defrag didn't stop."
                 }
 
-                # Test the the fragmentation is lower.
+                # Test the fragmentation is lower.
                 after 120 ;# serverCron only updates the info once in 100ms
                 set frag [s allocator_frag_ratio]
                 set max_latency 0
@@ -110,7 +110,7 @@ start_server {tags {"defrag"} overrides {appendonly yes auto-aof-rewrite-percent
                 }
             }
             # verify the data isn't corrupted or changed
-            set newdigest [r debug digest]
+            set newdigest [debug_digest]
             assert {$digest eq $newdigest}
             r save ;# saving an rdb iterates over all the data / pointers
 
@@ -118,7 +118,7 @@ start_server {tags {"defrag"} overrides {appendonly yes auto-aof-rewrite-percent
             if {[r config get activedefrag] eq "activedefrag yes"} {
                 # reset stats and load the AOF file
                 r config resetstat
-                r config set key-load-delay -25 ;# sleep on average 1/25 usec
+                r config set key-load-delay -50 ;# sleep on average 1/50 usec
                 r debug loadaof
                 r config set activedefrag no
                 # measure hits and misses right after aof loading
@@ -130,7 +130,7 @@ start_server {tags {"defrag"} overrides {appendonly yes auto-aof-rewrite-percent
                 set max_latency 0
                 foreach event [r latency latest] {
                     lassign $event eventname time latency max
-                    if {$eventname == "loading-cron"} {
+                    if {$eventname == "while-blocked-cron"} {
                         set max_latency $max
                     }
                 }
@@ -141,7 +141,7 @@ start_server {tags {"defrag"} overrides {appendonly yes auto-aof-rewrite-percent
                     puts "misses: $misses"
                     puts "max latency $max_latency"
                     puts [r latency latest]
-                    puts [r latency history loading-cron]
+                    puts [r latency history "while-blocked-cron"]
                 }
                 # make sure we had defrag hits during AOF loading
                 assert {$hits > 100000}
@@ -157,6 +157,88 @@ start_server {tags {"defrag"} overrides {appendonly yes auto-aof-rewrite-percent
         }
         r config set appendonly no
         r config set key-load-delay 0
+        
+        test "Active defrag eval scripts" {
+            r flushdb
+            r script flush sync
+            r config resetstat
+            r config set hz 100
+            r config set activedefrag no
+            r config set active-defrag-threshold-lower 5
+            r config set active-defrag-cycle-min 65
+            r config set active-defrag-cycle-max 75
+            r config set active-defrag-ignore-bytes 1500kb
+            r config set maxmemory 0
+            
+            set n 50000
+
+            # Populate memory with interleaving script-key pattern of same size
+            set dummy_script "--[string repeat x 400]\nreturn "
+            set rd [redis_deferring_client]
+            for {set j 0} {$j < $n} {incr j} {
+                set val "$dummy_script[format "%06d" $j]"
+                $rd script load $val
+                $rd set k$j $val
+            }
+            for {set j 0} {$j < $n} {incr j} {
+                $rd read ; # Discard script load replies
+                $rd read ; # Discard set replies
+            }
+            after 120 ;# serverCron only updates the info once in 100ms
+            if {$::verbose} {
+                puts "used [s allocator_allocated]"
+                puts "rss [s allocator_active]"
+                puts "frag [s allocator_frag_ratio]"
+                puts "frag_bytes [s allocator_frag_bytes]"
+            }                    
+            assert_lessthan [s allocator_frag_ratio] 1.05
+            
+            # Delete all the keys to create fragmentation
+            for {set j 0} {$j < $n} {incr j} { $rd del k$j }
+            for {set j 0} {$j < $n} {incr j} { $rd read } ; # Discard del replies
+            $rd close
+            after 120 ;# serverCron only updates the info once in 100ms
+            if {$::verbose} {
+                puts "used [s allocator_allocated]"
+                puts "rss [s allocator_active]"
+                puts "frag [s allocator_frag_ratio]"
+                puts "frag_bytes [s allocator_frag_bytes]"
+            }                    
+            assert_morethan [s allocator_frag_ratio] 1.4
+
+            catch {r config set activedefrag yes} e
+            if {[r config get activedefrag] eq "activedefrag yes"} {
+            
+                # wait for the active defrag to start working (decision once a second)
+                wait_for_condition 50 100 {
+                    [s active_defrag_running] ne 0
+                } else {
+                    fail "defrag not started."
+                }
+
+                # wait for the active defrag to stop working
+                wait_for_condition 500 100 {
+                    [s active_defrag_running] eq 0
+                } else {
+                    after 120 ;# serverCron only updates the info once in 100ms
+                    puts [r info memory]
+                    puts [r memory malloc-stats]
+                    fail "defrag didn't stop."
+                }
+
+                # test the fragmentation is lower
+                after 120 ;# serverCron only updates the info once in 100ms
+                if {$::verbose} {
+                    puts "used [s allocator_allocated]"
+                    puts "rss [s allocator_active]"
+                    puts "frag [s allocator_frag_ratio]"
+                    puts "frag_bytes [s allocator_frag_bytes]"
+                }                    
+                assert_lessthan_equal [s allocator_frag_ratio] 1.05
+            }                
+            # Flush all script to make sure we don't crash after defragging them
+            r script flush sync
+        } {OK}
 
         test "Active defrag big keys" {
             r flushdb
@@ -234,7 +316,7 @@ start_server {tags {"defrag"} overrides {appendonly yes auto-aof-rewrite-percent
             r config set latency-monitor-threshold 5
             r latency reset
 
-            set digest [r debug digest]
+            set digest [debug_digest]
             catch {r config set activedefrag yes} e
             if {[r config get activedefrag] eq "activedefrag yes"} {
                 # wait for the active defrag to start working (decision once a second)
@@ -254,7 +336,7 @@ start_server {tags {"defrag"} overrides {appendonly yes auto-aof-rewrite-percent
                     fail "defrag didn't stop."
                 }
 
-                # test the the fragmentation is lower
+                # test the fragmentation is lower
                 after 120 ;# serverCron only updates the info once in 100ms
                 set frag [s allocator_frag_ratio]
                 set max_latency 0
@@ -282,7 +364,7 @@ start_server {tags {"defrag"} overrides {appendonly yes auto-aof-rewrite-percent
                 }
             }
             # verify the data isn't corrupted or changed
-            set newdigest [r debug digest]
+            set newdigest [debug_digest]
             assert {$digest eq $newdigest}
             r save ;# saving an rdb iterates over all the data / pointers
         } {OK}
@@ -298,15 +380,15 @@ start_server {tags {"defrag"} overrides {appendonly yes auto-aof-rewrite-percent
             r config set active-defrag-cycle-max 75
             r config set active-defrag-ignore-bytes 2mb
             r config set maxmemory 0
-            r config set list-max-ziplist-size 1 ;# list of 100k items will have 100k quicklist nodes
+            r config set list-max-ziplist-size 5 ;# list of 500k items will have 100k quicklist nodes
 
             # create big keys with 10k items
             set rd [redis_deferring_client]
 
             set expected_frag 1.7
             # add a mass of list nodes to two lists (allocations are interlaced)
-            set val [string repeat A 500] ;# 1 item of 500 bytes puts us in the 640 bytes bin, which has 32 regs, so high potential for fragmentation
-            set elements 100000
+            set val [string repeat A 100] ;# 5 items of 100 bytes puts us in the 640 bytes bin, which has 32 regs, so high potential for fragmentation
+            set elements 500000
             for {set j 0} {$j < $elements} {incr j} {
                 $rd lpush biglist1 $val
                 $rd lpush biglist2 $val
@@ -330,7 +412,7 @@ start_server {tags {"defrag"} overrides {appendonly yes auto-aof-rewrite-percent
             r config set latency-monitor-threshold 5
             r latency reset
 
-            set digest [r debug digest]
+            set digest [debug_digest]
             catch {r config set activedefrag yes} e
             if {[r config get activedefrag] eq "activedefrag yes"} {
                 # wait for the active defrag to start working (decision once a second)
@@ -351,7 +433,7 @@ start_server {tags {"defrag"} overrides {appendonly yes auto-aof-rewrite-percent
                     fail "defrag didn't stop."
                 }
 
-                # test the the fragmentation is lower
+                # test the fragmentation is lower
                 after 120 ;# serverCron only updates the info once in 100ms
                 set misses [s active_defrag_misses]
                 set hits [s active_defrag_hits]
@@ -378,12 +460,12 @@ start_server {tags {"defrag"} overrides {appendonly yes auto-aof-rewrite-percent
                     assert {$max_latency <= 30}
                 }
 
-                # in extreme cases of stagnation, we see over 5m misses before the tests aborts with "defrag didn't stop",
-                # in normal cases we only see 100k misses out of 100k elements
-                assert {$misses < $elements * 2}
+                # in extreme cases of stagnation, we see over 20m misses before the tests aborts with "defrag didn't stop",
+                # in normal cases we only see 100k misses out of 500k elements
+                assert {$misses < $elements}
             }
             # verify the data isn't corrupted or changed
-            set newdigest [r debug digest]
+            set newdigest [debug_digest]
             assert {$digest eq $newdigest}
             r save ;# saving an rdb iterates over all the data / pointers
             r del biglist1 ;# coverage for quicklistBookmarksClear
@@ -393,7 +475,7 @@ start_server {tags {"defrag"} overrides {appendonly yes auto-aof-rewrite-percent
             # there was an edge case in defrag where all the slabs of a certain bin are exact the same
             # % utilization, with the exception of the current slab from which new allocations are made
             # if the current slab is lower in utilization the defragger would have ended up in stagnation,
-            # keept running and not move any allocation.
+            # kept running and not move any allocation.
             # this test is more consistent on a fresh server with no history
             start_server {tags {"defrag"} overrides {save ""}} {
                 r flushdb
@@ -450,7 +532,7 @@ start_server {tags {"defrag"} overrides {appendonly yes auto-aof-rewrite-percent
 
                 assert {$frag >= $expected_frag}
 
-                set digest [r debug digest]
+                set digest [debug_digest]
                 catch {r config set activedefrag yes} e
                 if {[r config get activedefrag] eq "activedefrag yes"} {
                     # wait for the active defrag to start working (decision once a second)
@@ -471,7 +553,7 @@ start_server {tags {"defrag"} overrides {appendonly yes auto-aof-rewrite-percent
                         fail "defrag didn't stop."
                     }
 
-                    # test the the fragmentation is lower
+                    # test the fragmentation is lower
                     after 120 ;# serverCron only updates the info once in 100ms
                     set misses [s active_defrag_misses]
                     set hits [s active_defrag_hits]
@@ -486,7 +568,7 @@ start_server {tags {"defrag"} overrides {appendonly yes auto-aof-rewrite-percent
                 }
 
                 # verify the data isn't corrupted or changed
-                set newdigest [r debug digest]
+                set newdigest [debug_digest]
                 assert {$digest eq $newdigest}
                 r save ;# saving an rdb iterates over all the data / pointers
             }
