@@ -1,7 +1,7 @@
-start_server {tags {"scan"}} {
+start_server {tags {"scan network"}} {
     test "SCAN basic" {
         r flushdb
-        r debug populate 1000
+        populate 1000
 
         set cur 0
         set keys {}
@@ -19,7 +19,7 @@ start_server {tags {"scan"}} {
 
     test "SCAN COUNT" {
         r flushdb
-        r debug populate 1000
+        populate 1000
 
         set cur 0
         set keys {}
@@ -37,7 +37,7 @@ start_server {tags {"scan"}} {
 
     test "SCAN MATCH" {
         r flushdb
-        r debug populate 1000
+        populate 1000
 
         set cur 0
         set keys {}
@@ -53,6 +53,51 @@ start_server {tags {"scan"}} {
         assert_equal 100 [llength $keys]
     }
 
+    test "SCAN TYPE" {
+        r flushdb
+        # populate only creates strings
+        populate 1000
+
+        # Check non-strings are excluded
+        set cur 0
+        set keys {}
+        while 1 {
+            set res [r scan $cur type "list"]
+            set cur [lindex $res 0]
+            set k [lindex $res 1]
+            lappend keys {*}$k
+            if {$cur == 0} break
+        }
+
+        assert_equal 0 [llength $keys]
+
+        # Check strings are included
+        set cur 0
+        set keys {}
+        while 1 {
+            set res [r scan $cur type "string"]
+            set cur [lindex $res 0]
+            set k [lindex $res 1]
+            lappend keys {*}$k
+            if {$cur == 0} break
+        }
+
+        assert_equal 1000 [llength $keys]
+
+        # Check all three args work together
+        set cur 0
+        set keys {}
+        while 1 {
+            set res [r scan $cur type "string" match "key:*" count 10]
+            set cur [lindex $res 0]
+            set k [lindex $res 1]
+            lappend keys {*}$k
+            if {$cur == 0} break
+        }
+
+        assert_equal 1000 [llength $keys]
+    }
+
     foreach enc {intset hashtable} {
         test "SSCAN with encoding $enc" {
             # Create the Set
@@ -69,7 +114,7 @@ start_server {tags {"scan"}} {
             r sadd set {*}$elements
 
             # Verify that the encoding matches.
-            assert {[r object encoding set] eq $enc}
+            assert_encoding $enc set
 
             # Test SSCAN
             set cur 0
@@ -87,11 +132,11 @@ start_server {tags {"scan"}} {
         }
     }
 
-    foreach enc {ziplist hashtable} {
+    foreach enc {listpack hashtable} {
         test "HSCAN with encoding $enc" {
             # Create the Hash
             r del hash
-            if {$enc eq {ziplist}} {
+            if {$enc eq {listpack}} {
                 set count 30
             } else {
                 set count 1000
@@ -103,7 +148,7 @@ start_server {tags {"scan"}} {
             r hmset hash {*}$elements
 
             # Verify that the encoding matches.
-            assert {[r object encoding hash] eq $enc}
+            assert_encoding $enc hash
 
             # Test HSCAN
             set cur 0
@@ -127,11 +172,11 @@ start_server {tags {"scan"}} {
         }
     }
 
-    foreach enc {ziplist skiplist} {
+    foreach enc {listpack skiplist} {
         test "ZSCAN with encoding $enc" {
             # Create the Sorted Set
             r del zset
-            if {$enc eq {ziplist}} {
+            if {$enc eq {listpack}} {
                 set count 30
             } else {
                 set count 1000
@@ -143,7 +188,7 @@ start_server {tags {"scan"}} {
             r zadd zset {*}$elements
 
             # Verify that the encoding matches.
-            assert {[r object encoding zset] eq $enc}
+            assert_encoding $enc zset
 
             # Test ZSCAN
             set cur 0
@@ -169,7 +214,7 @@ start_server {tags {"scan"}} {
 
     test "SCAN guarantees check under write load" {
         r flushdb
-        r debug populate 100
+        populate 100
 
         # We start scanning here, so keys from 0 to 99 should all be
         # reported at the end of the iteration.
