@@ -1,5 +1,8 @@
 start_server {tags {"slowlog"} overrides {slowlog-log-slower-than 1000000}} {
     test {SLOWLOG - check that it starts with an empty log} {
+        if {$::external} {
+            r slowlog reset
+        }
         r slowlog len
     } {0}
 
@@ -9,7 +12,7 @@ start_server {tags {"slowlog"} overrides {slowlog-log-slower-than 1000000}} {
         assert_equal [r slowlog len] 0
         r debug sleep 0.2
         assert_equal [r slowlog len] 1
-    }
+    } {} {needs:debug}
 
     test {SLOWLOG - max entries is correctly handled} {
         r config set slowlog-log-slower-than 0
@@ -35,11 +38,13 @@ start_server {tags {"slowlog"} overrides {slowlog-log-slower-than 1000000}} {
         r debug sleep 0.2
         set e [lindex [r slowlog get] 0]
         assert_equal [llength $e] 6
-        assert_equal [lindex $e 0] 105
+        if {!$::external} {
+            assert_equal [lindex $e 0] 105
+        }
         assert_equal [expr {[lindex $e 2] > 100000}] 1
         assert_equal [lindex $e 3] {debug sleep 0.2}
         assert_equal {foobar} [lindex $e 5]
-    }
+    } {} {needs:debug}
 
     test {SLOWLOG - Certain commands are omitted that contain sensitive information} {
         r config set slowlog-log-slower-than 0
@@ -48,18 +53,18 @@ start_server {tags {"slowlog"} overrides {slowlog-log-slower-than 1000000}} {
         r config set masterauth ""
         r acl setuser slowlog-test-user +get +set
         r config set slowlog-log-slower-than 0
-        r config set slowlog-log-slower-than 10000
+        r config set slowlog-log-slower-than -1
         set slowlog_resp [r slowlog get]
 
         # Make sure normal configs work, but the two sensitive
         # commands are omitted or redacted
         assert_equal 5 [llength $slowlog_resp]
-        assert_equal {slowlog reset} [lindex [lindex [r slowlog get] 4] 3]
-        assert_equal {acl setuser (redacted) (redacted) (redacted)} [lindex [lindex [r slowlog get] 3] 3]
-        assert_equal {config set masterauth (redacted)} [lindex [lindex [r slowlog get] 2] 3]
-        assert_equal {acl setuser (redacted) (redacted) (redacted)} [lindex [lindex [r slowlog get] 1] 3]
-        assert_equal {config set slowlog-log-slower-than 0} [lindex [lindex [r slowlog get] 0] 3]
-    }
+        assert_equal {slowlog reset} [lindex [lindex $slowlog_resp 4] 3]
+        assert_equal {acl setuser (redacted) (redacted) (redacted)} [lindex [lindex $slowlog_resp 3] 3]
+        assert_equal {config set masterauth (redacted)} [lindex [lindex $slowlog_resp 2] 3]
+        assert_equal {acl setuser (redacted) (redacted) (redacted)} [lindex [lindex $slowlog_resp 1] 3]
+        assert_equal {config set slowlog-log-slower-than 0} [lindex [lindex $slowlog_resp 0] 3]
+    } {} {needs:repl}
 
     test {SLOWLOG - Some commands can redact sensitive fields} {
         r config set slowlog-log-slower-than 0
@@ -67,14 +72,15 @@ start_server {tags {"slowlog"} overrides {slowlog-log-slower-than 1000000}} {
         r migrate [srv 0 host] [srv 0 port] key 9 5000
         r migrate [srv 0 host] [srv 0 port] key 9 5000 AUTH user
         r migrate [srv 0 host] [srv 0 port] key 9 5000 AUTH2 user password
+        r config set slowlog-log-slower-than -1
+        set slowlog_resp [r slowlog get]
 
-        r config set slowlog-log-slower-than 10000
         # Make sure all 3 commands were logged, but the sensitive fields are omitted
-        assert_equal 4 [llength [r slowlog get]]
-        assert_match {* key 9 5000} [lindex [lindex [r slowlog get] 2] 3]
-        assert_match {* key 9 5000 AUTH (redacted)} [lindex [lindex [r slowlog get] 1] 3]
-        assert_match {* key 9 5000 AUTH2 (redacted) (redacted)} [lindex [lindex [r slowlog get] 0] 3]
-    }
+        assert_equal 4 [llength $slowlog_resp]
+        assert_match {* key 9 5000} [lindex [lindex $slowlog_resp 2] 3]
+        assert_match {* key 9 5000 AUTH (redacted)} [lindex [lindex $slowlog_resp 1] 3]
+        assert_match {* key 9 5000 AUTH2 (redacted) (redacted)} [lindex [lindex $slowlog_resp 0] 3]
+    } {} {needs:repl}
 
     test {SLOWLOG - Rewritten commands are logged as their original command} {
         r config set slowlog-log-slower-than 0
@@ -113,11 +119,7 @@ start_server {tags {"slowlog"} overrides {slowlog-log-slower-than 1000000}} {
         # blocked BLPOP is replicated as LPOP
         set rd [redis_deferring_client]
         $rd blpop l 0
-        wait_for_condition 50 100 {
-            [s blocked_clients] eq {1}
-        } else {
-            fail "Clients are not blocked"
-        }
+        wait_for_blocked_clients_count 1 50 100
         r multi
         r lpush l foo
         r slowlog reset
@@ -131,7 +133,7 @@ start_server {tags {"slowlog"} overrides {slowlog-log-slower-than 1000000}} {
         r config set slowlog-log-slower-than 0
         r slowlog reset
         r sadd set 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33
-        set e [lindex [r slowlog get] 0]
+        set e [lindex [r slowlog get] end-1]
         lindex $e 3
     } {sadd set 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 {... (2 more arguments)}}
 
@@ -140,7 +142,7 @@ start_server {tags {"slowlog"} overrides {slowlog-log-slower-than 1000000}} {
         r slowlog reset
         set arg [string repeat A 129]
         r sadd set foo $arg
-        set e [lindex [r slowlog get] 0]
+        set e [lindex [r slowlog get] end-1]
         lindex $e 3
     } {sadd set foo {AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA... (1 more bytes)}}
 
@@ -154,7 +156,7 @@ start_server {tags {"slowlog"} overrides {slowlog-log-slower-than 1000000}} {
         assert_equal [r slowlog len] 1
         set e [lindex [r slowlog get] 0]
         assert_equal [lindex $e 3] {debug sleep 0.2}
-    }
+    } {} {needs:debug}
 
     test {SLOWLOG - can clean older entries} {
         r client setname lastentry_client
@@ -163,7 +165,7 @@ start_server {tags {"slowlog"} overrides {slowlog-log-slower-than 1000000}} {
         assert {[llength [r slowlog get]] == 1}
         set e [lindex [r slowlog get] 0]
         assert_equal {lastentry_client} [lindex $e 5]
-    }
+    } {} {needs:debug}
 
     test {SLOWLOG - can be disabled} {
         r config set slowlog-max-len 1
@@ -175,5 +177,27 @@ start_server {tags {"slowlog"} overrides {slowlog-log-slower-than 1000000}} {
         r slowlog reset
         r debug sleep 0.2
         assert_equal [r slowlog len] 0
+    } {} {needs:debug}
+
+    test {SLOWLOG - count must be >= -1} {
+       assert_error "ERR count should be greater than or equal to -1" {r slowlog get -2}
+       assert_error "ERR count should be greater than or equal to -1" {r slowlog get -222}
+    }
+
+    test {SLOWLOG - get all slow logs} {
+        r config set slowlog-log-slower-than 0
+        r config set slowlog-max-len 3
+        r slowlog reset
+
+        r set key test
+        r sadd set a b c
+        r incr num
+        r lpush list a
+
+        assert_equal [r slowlog len] 3
+        assert_equal 0 [llength [r slowlog get 0]]
+        assert_equal 1 [llength [r slowlog get 1]]
+        assert_equal 3 [llength [r slowlog get -1]]
+        assert_equal 3 [llength [r slowlog get 3]]
     }
 }
